@@ -17,22 +17,21 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-__id__        = "$Id: J2SE-access-bridge.py 2821 2007-09-12 14:28:07Z wwalker $"
-__version__   = "$Revision: 2821 $"
-__date__      = "$Date: 2007-09-12 10:28:07 -0400 (Wed, 12 Sep 2007) $"
+__id__        = "$Id: J2SE-access-bridge.py 3281 2007-11-29 20:05:13Z wwalker $"
+__version__   = "$Revision: 3281 $"
+__date__      = "$Date: 2007-11-29 15:05:13 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import atspi
+import pyatspi
+
 import default
 import debug
 import orca
 import orca_state
 import keybindings
-import settings
 import speech
 import rolenames
-import Accessibility
 import speechgenerator
 
 from orca_i18n import _ # for gettext support
@@ -60,33 +59,34 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances=[]
+        utterances = []
         if (not already_focused):
             utterances = self._getDefaultSpeech(obj, already_focused)
 
         obj.was_selected = False
-        if obj.state.count(Accessibility.STATE_EXPANDED) != 0:
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_EXPANDED):
             # Translators: this represents the state of a node in a tree.
             # 'expanded' means the children are showing.
             # 'collapsed' means the children are not showing.
             #
             utterances.append(_("expanded"))
-        elif obj.state.count(Accessibility.STATE_EXPANDED) == 0 and \
-                obj.state.count(Accessibility.STATE_EXPANDABLE) != 0:
+        elif not state.contains(pyatspi.STATE_EXPANDED) and \
+                 state.contains(pyatspi.STATE_EXPANDABLE):
             # Translators: this represents the state of a node in a tree.
             # 'expanded' means the children are showing.
             # 'collapsed' means the children are not showing.
             #
             utterances.append(_("collapsed"))
-        elif obj.state.count(Accessibility.STATE_SELECTED) != 0:
+        elif state.contains(pyatspi.STATE_SELECTED):
             # Translators: this represents the state of a node in a tree
             # or list.
             #
             # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
             utterances.append(Q_("listitem|selected"))
             obj.was_selected = True
-        elif obj.state.count(Accessibility.STATE_SELECTED) == 0 and \
-                obj.state.count(Accessibility.STATE_SELECTABLE) != 0:
+        elif not state.count(pyatspi.STATE_SELECTED) and \
+                 state.count(pyatspi.STATE_SELECTABLE):
             # Translators: this represents the state of a node in a tree
             # or list.
             #
@@ -140,8 +140,8 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         parent = obj.parent
         if parent \
-            and (obj.role == rolenames.ROLE_TABLE_CELL) \
-            and (parent.role == rolenames.ROLE_TABLE_CELL):
+            and (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
+            and (parent.getRole() == pyatspi.ROLE_TABLE_CELL):
             parent = parent.parent
 
         while parent and (parent.parent != parent):
@@ -154,12 +154,12 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 if text and len(text.strip()):
                     # Push announcement of cell to the end
                     #
-                    if not parent.role in [rolenames.ROLE_TABLE_CELL,
-                                           rolenames.ROLE_FILLER]:
+                    if not parent.getRole() in [pyatspi.ROLE_TABLE_CELL,
+                                                pyatspi.ROLE_FILLER]:
                         utterances.append(\
                             rolenames.getSpeechForRoleName(parent))
                     utterances.append(text)
-                    if parent.role == rolenames.ROLE_TABLE_CELL:
+                    if parent.getRole() == pyatspi.ROLE_TABLE_CELL:
                         utterances.append(\
                             rolenames.getSpeechForRoleName(parent))
             parent = parent.parent
@@ -213,30 +213,34 @@ class Script(default.Script):
         """
 
         self._debug("onFocus: %s '%s'" % \
-                    (event.source.role, event.source.name))
+                    (event.source.getRole(), event.source.name))
 
-        role = event.source.role
-        if role == rolenames.ROLE_LIST:
+        role = event.source.getRole()
+        if role == pyatspi.ROLE_LIST:
             selectedItem = None
-            selection = event.source.selection
+            try:
+                selection = event.source.querySelection()
+            except:
+                selection = None
+
             if selection and selection.nSelectedChildren > 0:
-                selectedItem = atspi.Accessible.makeAccessible(
-                                        selection.getSelectedChild(0))
+                selectedItem = selection.getSelectedChild(0)
             elif event.source.childCount > 0:
-                selectedItem = event.source.child(0)
+                selectedItem = event.getChildAtIndex(0)
             if selectedItem:
                 orca.setLocusOfFocus(event, selectedItem)
             else:
-                # if impossible to get selection or list has 0 items, present list
+                # if impossible to get selection or list has 0 items,
+                # present list
                 orca.setLocusOfFocus(event, event.source)
-        elif role == rolenames.ROLE_LABEL:
+        elif role == pyatspi.ROLE_LABEL:
             # In FileChooserDemo, when enter in a new folder, a focus event for
             # the top combo box selected item (not SHOWING item) is received.
             # Should this check be more specific ?
             #
-            if not event.source.state.count (Accessibility.STATE_SHOWING):
+            if not event.source.getState().contains(pyatspi.STATE_SHOWING):
                 return
-        elif role == rolenames.ROLE_MENU:
+        elif role == pyatspi.ROLE_MENU:
             # A JMenu has always selection.nSelectedChildren > 0
             orca.setLocusOfFocus(event, event.source)
         else:
@@ -251,7 +255,8 @@ class Script(default.Script):
         """
 
         self._debug("onStateChanged: %s: %s '%s' (%d,%d)" % \
-                    (event.type, event.source.role, event.source.name, event.detail1, event.detail2))
+                    (event.type, event.source.getRole(), 
+                     event.source.name, event.detail1, event.detail2))
 
         # This is a workaround for a java-access-bridge bug (Bug 355011)
         # where popup menu events are not sent to Orca.
@@ -260,20 +265,21 @@ class Script(default.Script):
         # If there is a popup menu, give locus of focus to the armed menu
         # item.
         #
-        if event.source.role == rolenames.ROLE_ROOT_PANE and \
+        if event.source.getRole() == pyatspi.ROLE_ROOT_PANE and \
                event.type.startswith("object:state-changed:focused") and \
                event.detail1 == 1:
 
-            for i in range(0, event.source.childCount):
+            for child in event.source:
                 # search the layered pane for a popup menu
-                child = event.source.child(i)
-                if child.role == rolenames.ROLE_LAYERED_PANE:
-                    popup = self.findByRole(child, rolenames.ROLE_POPUP_MENU, False)
+                if child.getRole() == pyatspi.ROLE_LAYERED_PANE:
+                    popup = self.findByRole(child, 
+                                            pyatspi.ROLE_POPUP_MENU, False)
                     if len(popup) > 0:
                         # set the locus of focus to the armed menu item
-                        items = self.findByRole(popup[0], rolenames.ROLE_MENU_ITEM, False)
+                        items = self.findByRole(popup[0], 
+                                                pyatspi.ROLE_MENU_ITEM, False)
                         for item in items:
-                            if item.state.count(Accessibility.STATE_ARMED):
+                            if item.getState().contains(pyatspi.STATE_ARMED):
                                 orca.setLocusOfFocus(event, item)
                                 return
 
@@ -287,7 +293,7 @@ class Script(default.Script):
         # Hand state changes when JTree labels become expanded
         # or collapsed.
         #
-        if (event.source.role == rolenames.ROLE_LABEL) and \
+        if (event.source.getRole() == pyatspi.ROLE_LABEL) and \
             event.type.startswith("object:state-changed:expanded"):
             orca.visualAppearanceChanged(event, event.source)
             return
@@ -296,8 +302,8 @@ class Script(default.Script):
         # Fix for Swing file chooser.
         #
         if event.type.startswith("object:state-changed:visible") and \
-                event.source.role == rolenames.ROLE_POPUP_MENU and \
-                event.source.parent.state.count(Accessibility.STATE_FOCUSED):
+                event.source.getRole() == pyatspi.ROLE_POPUP_MENU and \
+                event.source.parent.getState().contains(pyatspi.STATE_FOCUSED):
             orca.setLocusOfFocus(event, event.source.parent)
             return
 
@@ -311,43 +317,43 @@ class Script(default.Script):
         """
 
         self._debug("onSelectionChanged: %s '%s'" % \
-                    (event.source.role, event.source.name))
+                    (event.source.getRole(), event.source.name))
 
-        if not event.source.state.count (atspi.Accessibility.STATE_FOCUSED):
+        if not event.source.getState().contains(pyatspi.STATE_FOCUSED):
             return
 
-        if event.source.role == rolenames.ROLE_TABLE:
+        if event.source.getRole() == pyatspi.ROLE_TABLE:
             return
 
-        if event.source.role == rolenames.ROLE_TREE:
+        if event.source.getRole() == pyatspi.ROLE_TREE:
             return
 
-        if event.source.role == rolenames.ROLE_LIST:
-            selection = event.source.selection
+        if event.source.getRole() == pyatspi.ROLE_LIST:
+            selection = event.source.querySelection()
             if selection.nSelectedChildren <= 0:
                 if event.source.childCount > 0:
-                    child = event.source.child(0)
-                    if child.state.count (atspi.Accessibility.STATE_ACTIVE):
+                    child = event.source[0]
+                    if child.getState().contains(pyatspi.STATE_ACTIVE):
                         return
                 else:
                     orca.setLocusOfFocus(event, event.source)
                     return
             else:
-                selectedItem = atspi.Accessible.makeAccessible(
-                                        selection.getSelectedChild(0))
+                selectedItem = selection.getSelectedChild(0)
 
-                # If the selected list item is the same with the last focused object,
-                # present it only if the item was not selected and has SELECTED state
-                # or if the item doesn't have SELECTED state, but SELECTABLE and
-                # it was selected.
+                # If the selected list item is the same with the last 
+                # focused object, present it only if the item was not 
+                # selected and has SELECTED state or if the item doesn't 
+                # have SELECTED state, but SELECTABLE and it was selected.
                 #
                 if orca_state.activeScript and \
                    orca_state.activeScript.isSameObject(selectedItem,  \
                                                     orca_state.locusOfFocus):
-                    if (orca_state.locusOfFocus.state.count(Accessibility.STATE_SELECTED) != 0 \
-                                and not orca_state.locusOfFocus.was_selected)   \
-                        or (orca_state.locusOfFocus.state.count(Accessibility.STATE_SELECTED) == 0 \
-                                and orca_state.locusOfFocus.state.count(Accessibility.STATE_SELECTABLE) != 0 \
+                    state = orca_state.locusOfFocus.getState()
+                    if (state.contains(pyatspi.STATE_SELECTED) \
+                              and not orca_state.locusOfFocus.was_selected)   \
+                        or (not state.contains(pyatspi.STATE_SELECTED) \
+                          and state.contains(pyatspi.STATE_SELECTABLE) \
                                 and orca_state.locusOfFocus.was_selected):
                         orca.visualAppearanceChanged(event, selectedItem)
                     return
@@ -363,13 +369,13 @@ class Script(default.Script):
         """
 
         self._debug("onValueChanged: %s '%s' value=%d" % \
-                    (event.source.role, event.source.name,
-                     event.source.value.currentValue))
+                    (event.source.getRole(), event.source.name,
+                     event.source.queryValue().currentValue))
 
         # We'll let state-changed:checked event to be used to
         # manage check boxes in java application
         #
-        if event.source.role == rolenames.ROLE_CHECK_BOX and \
+        if event.source.getRole() == pyatspi.ROLE_CHECK_BOX and \
                event.type.startswith("object:property-change:accessible-value"):
             return
 
@@ -390,7 +396,8 @@ class Script(default.Script):
         """
 
         self._debug("visualAppearanceChanged: %s '%s', locusOfFocus=%s" % \
-                    (obj.role, obj.name, orca_state.locusOfFocus.role))
+                    (obj.getRole(), obj.name, 
+                     orca_state.locusOfFocus.getRole()))
 
         # Present to Braille and Speech an object that is the same
         # with the last focused object, only if it is a label.
@@ -398,17 +405,22 @@ class Script(default.Script):
         # (see Open dialog lists).
         #
         if orca_state.activeScript.isSameObject(obj, orca_state.locusOfFocus):
-            if obj.role == rolenames.ROLE_LABEL:
+            if obj.getRole() == pyatspi.ROLE_LABEL:
                 self.updateBraille(orca_state.locusOfFocus)
-                speech.speakUtterances(self.speechGenerator.getSpeech(orca_state.locusOfFocus, True))
+                speech.speakUtterances( \
+                  self.speechGenerator.getSpeech(orca_state.locusOfFocus, True))
                 return
 
-        if obj.role == rolenames.ROLE_SPIN_BOX:
+        if obj.getRole() == rolenames.ROLE_SPIN_BOX:
             # Check for the spinbox text object being null. There appears
             # to be a bug where the text object for some text fields
             # is null. 
-            if orca_state.locusOfFocus.role == rolenames.ROLE_TEXT and \
-               orca_state.locusOfFocus.text == None:
+            try:
+                text = orca_state.locusOfFocus.queryText()
+            except:
+                text = None
+            if orca_state.locusOfFocus.getRole() == pyatspi.ROLE_TEXT and \
+               text == None:
                 # Set the locusOfFocus to the spinbox itself.
                 orca.setLocusOfFocus(event, obj)
 
@@ -422,12 +434,17 @@ class Script(default.Script):
         """
 
         self._debug("onCaretMoved: %s '%s'" % \
-                    (event.source.role, event.source.name))
+                    (event.source.getRole(), event.source.name))
 
         # Check for bogus events containing null text objects.
-        if event.source.text == None:
+        try:
+            text = event.source.queryText()
+        except:
+            text = None
+
+        if text == None:
             self._debug("bogus event: text object is null. Discarding event")
-            return;
+            return
 
         default.Script.onCaretMoved(self, event)
 
@@ -439,30 +456,34 @@ class Script(default.Script):
         """
 
         self._debug("onTextInserted: %s '%s'" % \
-                    (event.source.role, event.source.name))
+                    (event.source.getRole(), event.source.name))
 
         # Check for bogus events containing null text objects.
-        if event.source.text == None:
+        try:
+            text = event.source.queryText()
+        except:
+            text = None
+        if text == None:
             self._debug("bogus event: text object is null. Discarding event")
-            return;
+            return
 
         default.Script.onTextInserted(self, event)        
 
-    def getDisplayedLabel(self, object):
+    def getDisplayedLabel(self, obj):
         """If there is an object labelling the given object, return the
         text being displayed for the object labelling this object.
         Otherwise, return None.
 
         Argument:
-        - object: the object in question
+        - obj: the object in question
 
         Returns the string of the object labelling this object, or None
         if there is nothing of interest here.
         """
-        label = default.Script.getDisplayedLabel(self, object)
+        label = default.Script.getDisplayedLabel(self, obj)
         if not label and \
-           (object.role == rolenames.ROLE_TEXT or \
-            object.role == rolenames.ROLE_COMBO_BOX):
+           (obj.getRole() == pyatspi.ROLE_TEXT or \
+            obj.getRole() == pyatspi.ROLE_COMBO_BOX):
             # This is a workaround for a common Java accessibility
             # bug where a label next to a component is used to
             # label the component. This relation is only obvious to
@@ -473,10 +494,10 @@ class Script(default.Script):
             # ends with a colon. We only do this for text and combo
             # boxes since they are the most common Java component with
             # this bug.
-            if object.index > 0:
-                prevSibling = object.parent.child(object.index-1)
+            if obj.getIndexInParent() > 0:
+                prevSibling = obj.parent[obj.getIndexInParent()-1]
                 if prevSibling and prevSibling.name and \
-                   prevSibling.role == rolenames.ROLE_LABEL:
+                   prevSibling.getRole() == pyatspi.ROLE_LABEL:
                     labelFor = prevSibling.name.rstrip(" ")
                     if (labelFor.endswith(":")):
                         return labelFor.rstrip(":")
