@@ -37,6 +37,7 @@ import default
 import focus_tracking_presenter
 import input_event
 import keybindings
+import orca_glade
 import orca_gui_prefs
 import orca_state
 import platform
@@ -47,11 +48,29 @@ from orca_i18n import _  # for gettext support
 
 applicationName = None
 appScript = None
-OS = None
 
-class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
+class OrcaSetupGUI(orca_gui_prefs.OrcaSetupGUI):
 
-    def _initAppGUIState(self, thisAppScript):
+    def __init__(self, fileName, windowName):
+        """Initialize the application specific Orca configuration GUI.
+
+        Arguments:
+        - fileName: name of the Glade file.
+        - windowName: name of the component to get from the Glade file.
+        """
+
+        orca_gui_prefs.OrcaSetupGUI.__init__(self, fileName, windowName)
+
+        # Initialize variables to None to keep pylint happy.
+        #
+        self.applicationName = None
+        self.appScript = None
+        self.app = None
+        self.kbindings = None
+        self.appKeyBindings = None
+        self.defKeyBindings = None
+
+    def initAppGUIState(self, thisAppScript):
         """Before we show the GUI to the user we want to remove the
         General tab and gray out the Speech systems and servers 
         controls on the speech tab.
@@ -86,10 +105,10 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
           pronunciation from.
         """
 
-        orca_gui_prefs.orcaSetupGUI._createPronunciationTreeView( \
+        orca_gui_prefs.OrcaSetupGUI._createPronunciationTreeView( \
                               self, appScript.app_pronunciation_dict)
 
-    def _showGUI(self):
+    def showGUI(self):
         """Show the app-specific Orca configuration GUI window. This 
         assumes that the GUI has already been created.
         """
@@ -102,7 +121,7 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         title = _("Orca Preferences for %s") % self.applicationName
         self.get_widget("orcaSetupWindow").set_title(title)
 
-        orca_gui_prefs.orcaSetupGUI._showGUI(self)
+        orca_gui_prefs.OrcaSetupGUI.showGUI(self)
 
     def writeUserPreferences(self):
         """Write out the user's application-specific Orca preferences.
@@ -118,8 +137,6 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         """ Mark as modified the user application specific custom key bindings:
         """
 
-        global appScript
-
         try:
             appScript.setupInputEventHandlers()
             keyBinds = keybindings.KeyBindings()
@@ -155,8 +172,6 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         if clearModel:
             self.keyBindingsModel.clear()
 
-        global applicationName, appScript
-
         # Get the key bindings for the application script.
         #
         self.appKeyBindings = appScript.getKeyBindings()
@@ -197,7 +212,7 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         # setting up the tree view.
         #
         self.kbindings = self.appKeyBindings
-        orca_gui_prefs.orcaSetupGUI._populateKeyBindings(self, False)
+        orca_gui_prefs.OrcaSetupGUI._populateKeyBindings(self, False)
 
     def okButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the okButton
@@ -217,20 +232,52 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
 
     def windowDestroyed(self, widget):
         """Signal handler for the "destroyed" signal for the orcaSetupWindow
-           GtkWindow widget. Reset OS to None, so that the GUI can be rebuilt
-           from the Glade file the next time the user wants to display the
-           configuration GUI.
+           GtkWindow widget. Reset orca_state.appOS to None, so that the 
+           GUI can be rebuilt from the Glade file the next time the user 
+           wants to display the configuration GUI.
 
         Arguments:
         - widget: the component that generated the signal.
         """
 
-        global OS
+        orca_state.appOS = None
 
-        OS = None
+class WarningDialogGUI(orca_glade.GladeWrapper):
+
+    def getPrefsWarningDialog(self):
+        """Return a handle to the Orca Preferences warning dialog.
+        """
+
+        return self.orcaPrefsWarningDialog
+
+    def orcaPrefsWarningDialogDestroyed(self, widget):
+        """Signal handler for the "destroyed" signal for the 
+        orcaPrefsWarningDialog GtkWindow widget. Reset orca_state.orcaWD
+        to None, so that the GUI can be rebuilt from the Glade file the 
+        next time that this warning dialog has to be displayed.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        orca_state.orcaWD = None
+
+    def orcaPrefsWarningDialogOKButtonClicked(self, widget):
+        """Signal handler for the "clicked" signal for the
+        orcaPrefsWarningDialogOKButton GtkButton widget. The user has clicked
+        the OK button in the Orca Preferences warning dialog.
+        This dialog informs the user that they already have an instance 
+        of an Orca preferences dialog open, and that they will need to 
+        close it before opening a new one.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.orcaPrefsWarningDialog.destroy()
 
 def showPreferencesUI():
-    global applicationName, appScript, OS
+    global applicationName, appScript
 
     # There must be an application with focus for this to work.
     #
@@ -247,29 +294,39 @@ def showPreferencesUI():
     #
     applicationName = orca_state.locusOfFocus.getApplication().name
 
-    # Translators: Orca Preferences in this case, is a configuration GUI 
-    # for allowing users to set application specific settings from within
-    # Orca for the application that currently has focus.
-    #
-    line = _("Starting Orca Preferences for %s. This may take a while.") % \
-           applicationName
-    braille.displayMessage(line)
-    speech.speak(line)
-
     removeGeneralPane = False
-    if not OS:
-        gladeFile = os.path.join(platform.prefix,
-                                 platform.datadirname,
-                                 platform.package,
-                                 "glade",
-                                 "orca-setup.glade")
-        OS = orcaSetupGUI(gladeFile, "orcaSetupWindow")
+    if not orca_state.appOS and not orca_state.orcaOS:
+        # Translators: Orca Preferences in this case, is a configuration GUI
+        # for allowing users to set application specific settings from within
+        # Orca for the application that currently has focus.
+        #
+        line = _("Starting Orca Preferences for %s. This may take a while.") % \
+               applicationName
+        braille.displayMessage(line)
+        speech.speak(line)
+
+        orca_state.prefsGladeFile = os.path.join(platform.prefix,
+                                                 platform.datadirname,
+                                                 platform.package,
+                                                 "glade",
+                                                 "orca-setup.glade")
+        orca_state.appOS = OrcaSetupGUI(orca_state.prefsGladeFile,
+                                        "orcaSetupWindow")
         removeGeneralPane = True
+        orca_state.appOS.init()
+    else:
+        if not orca_state.orcaWD:
+            orca_state.orcaWD = \
+                WarningDialogGUI(orca_state.prefsGladeFile,
+                                 "orcaPrefsWarningDialog")
+            warningDialog = orca_state.orcaWD.getPrefsWarningDialog()
+            warningDialog.realize()
+            warningDialog.show()
+        return
 
-    OS._init()
     if removeGeneralPane:
-        OS._initAppGUIState(appScript)
-    OS._showGUI()
+        orca_state.appOS.initAppGUIState(appScript)
+    orca_state.appOS.showGUI()
 
 def main():
     locale.setlocale(locale.LC_ALL, '')
