@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 3190 2007-10-26 18:48:24Z shaeger $"
-__version__   = "$Revision: 3190 $"
-__date__      = "$Date: 2007-10-26 14:48:24 -0400 (Fri, 26 Oct 2007) $"
+__id__        = "$Id: flat_review.py 3313 2007-11-30 21:23:49Z richb $"
+__version__   = "$Revision: 3313 $"
+__date__      = "$Date: 2007-11-30 16:23:49 -0500 (Fri, 30 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -204,16 +204,17 @@ class Zone:
             return False
 
     def getWordAtOffset(self, charOffset):
-        word = None
+        wordAtOffset = None
         offset = 0
         for word in self.words:
             nextOffset = offset + len(word.string.decode("UTF-8"))
+            wordAtOffset = word
             if nextOffset > charOffset:
-                return [word, charOffset - offset]
+                return [wordAtOffset, charOffset - offset]
             else:
                 offset = nextOffset
 
-        return [word, offset]
+        return [wordAtOffset, offset]
 
 class TextZone(Zone):
     """Represents Accessibility_Text that is a portion of a single
@@ -299,14 +300,20 @@ class StateZone(Zone):
         if attr in ["string", "length", "brailleString"]:
             # stateCount is used as a boolean and as an index
             stateset = self.accessible.getState()
-            if stateset.contains(pyatspi.STATE_CHECKED):
+            if stateset.contains(pyatspi.STATE_INDETERMINATE):
+                stateCount = 2
+            elif stateset.contains(pyatspi.STATE_CHECKED):
                 stateCount = 1
             else:
                 stateCount = 0
             if self.role in [pyatspi.ROLE_CHECK_BOX,
                              pyatspi.ROLE_CHECK_MENU_ITEM,
                              pyatspi.ROLE_TABLE_CELL]:
-                if stateCount:
+                if stateCount == 2:
+                    # Translators: this represents the state of a checkbox.
+                    #
+                    speechState = _("partially checked")
+                elif stateCount == 1:
                     # Translators: this represents the state of a checkbox.
                     #
                     speechState = _("checked")
@@ -547,6 +554,11 @@ class Context:
     WRAP_TOP_BOTTOM = 1 << 1
     WRAP_ALL        = (WRAP_LINE | WRAP_TOP_BOTTOM)
 
+    # A minimal chunk to jump around should we not really know where we
+    # are going.
+    #
+    GRID_SIZE = 7
+
     def __init__(self, script):
         """Create a new Context that will be used for handling flat
         review mode.
@@ -1157,11 +1169,6 @@ class Context:
         except NotImplementedError:
             return []
 
-        # A minimal chunk to jump around should we not really know where we
-        # are going.
-        #
-        GRID_SIZE = 7
-
         descendants = []
 
         parentExtents = icomponent.getExtents(0)
@@ -1217,15 +1224,15 @@ class Context:
                         if not descendants.count(child):
                             descendants.append(child)
                     else:
-                        newX = currentX + GRID_SIZE
+                        newX = currentX + self.GRID_SIZE
                 else:
-                    newX = currentX + GRID_SIZE
+                    newX = currentX + self.GRID_SIZE
                 if newX <= currentX:
-                    currentX += GRID_SIZE
+                    currentX += self.GRID_SIZE
                 else:
                     currentX = newX
             if minHeight == sys.maxint:
-                minHeight = GRID_SIZE
+                minHeight = self.GRID_SIZE
             currentY += minHeight
 
         return descendants
@@ -1499,12 +1506,12 @@ class Context:
 
         return zone.accessible
 
-    def getCurrent(self, type=ZONE):
+    def getCurrent(self, flatReviewType=ZONE):
         """Gets the string, offset, and extent information for the
         current locus of interest.
 
         Arguments:
-        - type: one of ZONE, CHAR, WORD, LINE
+        - flatReviewType: one of ZONE, CHAR, WORD, LINE
 
         Returns: [string, x, y, width, height]
         """
@@ -1515,13 +1522,13 @@ class Context:
 
         zone = self.lines[self.lineIndex].zones[self.zoneIndex]
 
-        if type == Context.ZONE:
+        if flatReviewType == Context.ZONE:
             return [zone.string,
                     zone.x,
                     zone.y,
                     zone.width,
                     zone.height]
-        elif type == Context.CHAR:
+        elif flatReviewType == Context.CHAR:
             if isinstance(zone, TextZone):
                 words = zone.words
                 if words:
@@ -1541,7 +1548,7 @@ class Context:
                                 word.width,
                                 word.height]
             return self.getCurrent(Context.ZONE)
-        elif type == Context.WORD:
+        elif flatReviewType == Context.WORD:
             if isinstance(zone, TextZone):
                 words = zone.words
                 if words:
@@ -1552,7 +1559,7 @@ class Context:
                             word.width,
                             word.height]
             return self.getCurrent(Context.ZONE)
-        elif type == Context.LINE:
+        elif flatReviewType == Context.LINE:
             line = self.lines[self.lineIndex]
             return [line.string,
                     line.x,
@@ -1560,7 +1567,7 @@ class Context:
                     line.width,
                     line.height]
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
     def getCurrentBrailleRegions(self):
         """Gets the braille for the entire current line.
@@ -1592,24 +1599,24 @@ class Context:
 
         return [regions, regionWithFocus]
 
-    def goBegin(self, type=WINDOW):
+    def goBegin(self, flatReviewType=WINDOW):
         """Moves this context's locus of interest to the first char
         of the first relevant zone.
 
         Arguments:
-        - type: one of ZONE, LINE or WINDOW
+        - flatReviewType: one of ZONE, LINE or WINDOW
 
         Returns True if the locus of interest actually changed.
         """
 
-        if (type == Context.LINE) or (type == Context.ZONE):
+        if (flatReviewType == Context.LINE) or (flatReviewType == Context.ZONE):
             lineIndex = self.lineIndex
-        elif type == Context.WINDOW:
+        elif flatReviewType == Context.WINDOW:
             lineIndex = 0
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
-        if type == Context.ZONE:
+        if flatReviewType == Context.ZONE:
             zoneIndex = self.zoneIndex
         else:
             zoneIndex = 0
@@ -1631,24 +1638,24 @@ class Context:
 
         return moved
 
-    def goEnd(self, type=WINDOW):
+    def goEnd(self, flatReviewType=WINDOW):
         """Moves this context's locus of interest to the last char
         of the last relevant zone.
 
         Arguments:
-        - type: one of ZONE, LINE, or WINDOW
+        - flatReviewType: one of ZONE, LINE, or WINDOW
 
         Returns True if the locus of interest actually changed.
         """
 
-        if (type == Context.LINE) or (type == Context.ZONE):
+        if (flatReviewType == Context.LINE) or (flatReviewType == Context.ZONE):
             lineIndex = self.lineIndex
-        elif type == Context.WINDOW:
+        elif flatReviewType == Context.WINDOW:
             lineIndex  = len(self.lines) - 1
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
-        if type == Context.ZONE:
+        if flatReviewType == Context.ZONE:
             zoneIndex = self.zoneIndex
         else:
             zoneIndex = len(self.lines[lineIndex].zones) - 1
@@ -1679,12 +1686,13 @@ class Context:
 
         return moved
 
-    def goPrevious(self, type=ZONE, wrap=WRAP_ALL, omitWhitespace=True):
+    def goPrevious(self, flatReviewType=ZONE, 
+                   wrap=WRAP_ALL, omitWhitespace=True):
         """Moves this context's locus of interest to the first char
         of the previous type.
 
         Arguments:
-        - type: one of ZONE, CHAR, WORD, LINE
+        - flatReviewType: one of ZONE, CHAR, WORD, LINE
         - wrap: if True, will cross boundaries, including top and
                 bottom; if False, will stop on boundaries.
 
@@ -1693,7 +1701,7 @@ class Context:
 
         moved = False
 
-        if type == Context.ZONE:
+        if flatReviewType == Context.ZONE:
             if self.zoneIndex > 0:
                 self.zoneIndex -= 1
                 self.wordIndex = 0
@@ -1712,7 +1720,7 @@ class Context:
                     self.wordIndex = 0
                     self.charIndex = 0
                     moved = True
-        elif type == Context.CHAR:
+        elif flatReviewType == Context.CHAR:
             if self.charIndex > 0:
                 self.charIndex -= 1
                 moved = True
@@ -1724,7 +1732,7 @@ class Context:
                         chars = zone.words[self.wordIndex].chars
                         if chars:
                             self.charIndex = len(chars) - 1
-        elif type == Context.WORD:
+        elif flatReviewType == Context.WORD:
             zone = self.lines[self.lineIndex].zones[self.zoneIndex]
             accessible = zone.accessible
             lineIndex = self.lineIndex
@@ -1781,7 +1789,7 @@ class Context:
                 self.wordIndex = wordIndex
                 self.charIndex = charIndex
 
-        elif type == Context.LINE:
+        elif flatReviewType == Context.LINE:
             if wrap & Context.WRAP_LINE:
                 if self.lineIndex > 0:
                     self.lineIndex -= 1
@@ -1797,26 +1805,26 @@ class Context:
                     self.charIndex = 0
                     moved = True
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
-        if moved and (type != Context.LINE):
+        if moved and (flatReviewType != Context.LINE):
             self.targetCharInfo = self.getCurrent(Context.CHAR)
 
         return moved
 
-    def goNext(self, type=ZONE, wrap=WRAP_ALL, omitWhitespace=True):
+    def goNext(self, flatReviewType=ZONE, wrap=WRAP_ALL, omitWhitespace=True):
         """Moves this context's locus of interest to first char of
         the next type.
 
         Arguments:
-        - type: one of ZONE, CHAR, WORD, LINE
+        - flatReviewType: one of ZONE, CHAR, WORD, LINE
         - wrap: if True, will cross boundaries, including top and
                 bottom; if False, will stop on boundaries.
         """
 
         moved = False
 
-        if type == Context.ZONE:
+        if flatReviewType == Context.ZONE:
             if self.zoneIndex < (len(self.lines[self.lineIndex].zones) - 1):
                 self.zoneIndex += 1
                 self.wordIndex = 0
@@ -1835,7 +1843,7 @@ class Context:
                     self.wordIndex = 0
                     self.charIndex = 0
                     moved = True
-        elif type == Context.CHAR:
+        elif flatReviewType == Context.CHAR:
             zone = self.lines[self.lineIndex].zones[self.zoneIndex]
             if zone.words:
                 chars = zone.words[self.wordIndex].chars
@@ -1849,7 +1857,7 @@ class Context:
                     moved = self.goNext(Context.WORD, wrap)
             else:
                 moved = self.goNext(Context.ZONE, wrap)
-        elif type == Context.WORD:
+        elif flatReviewType == Context.WORD:
             zone = self.lines[self.lineIndex].zones[self.zoneIndex]
             accessible = zone.accessible
             lineIndex = self.lineIndex
@@ -1905,7 +1913,7 @@ class Context:
                 self.wordIndex = wordIndex
                 self.charIndex = charIndex
 
-        elif type == Context.LINE:
+        elif flatReviewType == Context.LINE:
             if wrap & Context.WRAP_LINE:
                 if self.lineIndex < (len(self.lines) - 1):
                     self.lineIndex += 1
@@ -1921,20 +1929,20 @@ class Context:
                     self.charIndex = 0
                     moved = True
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
-        if moved and (type != Context.LINE):
+        if moved and (flatReviewType != Context.LINE):
             self.targetCharInfo = self.getCurrent(Context.CHAR)
 
         return moved
 
-    def goAbove(self, type=LINE, wrap=WRAP_ALL):
+    def goAbove(self, flatReviewType=LINE, wrap=WRAP_ALL):
         """Moves this context's locus of interest to first char
         of the type that's closest to and above the current locus of
         interest.
 
         Arguments:
-        - type: LINE
+        - flatReviewType: LINE
         - wrap: if True, will cross top/bottom boundaries; if False, will
                 stop on top/bottom boundaries.
 
@@ -1942,7 +1950,7 @@ class Context:
         """
 
         moved = False
-        if type == Context.CHAR:
+        if flatReviewType == Context.CHAR:
             # We want to shoot for the closest character, which we've
             # saved away as self.targetCharInfo, which is the list
             # [string, x, y, width, height].
@@ -1968,20 +1976,20 @@ class Context:
             # so we reset it to our saved value.
             #
             self.targetCharInfo = target
-        elif type == Context.LINE:
-            return self.goPrevious(type, wrap)
+        elif flatReviewType == Context.LINE:
+            return self.goPrevious(flatReviewType, wrap)
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
         return moved
 
-    def goBelow(self, type=LINE, wrap=WRAP_ALL):
+    def goBelow(self, flatReviewType=LINE, wrap=WRAP_ALL):
         """Moves this context's locus of interest to the first
         char of the type that's closest to and below the current
         locus of interest.
 
         Arguments:
-        - type: one of WORD, LINE
+        - flatReviewType: one of WORD, LINE
         - wrap: if True, will cross top/bottom boundaries; if False, will
                 stop on top/bottom boundaries.
 
@@ -1989,7 +1997,7 @@ class Context:
         """
 
         moved = False
-        if type == Context.CHAR:
+        if flatReviewType == Context.CHAR:
             # We want to shoot for the closest character, which we've
             # saved away as self.targetCharInfo, which is the list
             # [string, x, y, width, height].
@@ -2015,9 +2023,9 @@ class Context:
             # so we reset it to our saved value.
             #
             self.targetCharInfo = target
-        elif type == Context.LINE:
-            moved = self.goNext(type, wrap)
+        elif flatReviewType == Context.LINE:
+            moved = self.goNext(flatReviewType, wrap)
         else:
-            raise Exception("Invalid type: %d" % type)
+            raise Exception("Invalid type: %d" % flatReviewType)
 
         return moved
