@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__  = "$Id: focus_tracking_presenter.py 3192 2007-10-26 19:41:18Z eitani $"
-__version__   = "$Revision: 3192 $"
-__date__      = "$Date: 2007-10-26 15:41:18 -0400 (Fri, 26 Oct 2007) $"
+__id__  = "$Id: focus_tracking_presenter.py 3324 2007-12-02 22:19:15Z wwalker $"
+__version__   = "$Revision: 3324 $"
+__date__      = "$Date: 2007-12-02 17:19:15 -0500 (Sun, 02 Dec 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -52,6 +52,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
     def __init__(self):
 
+        presentation_manager.PresentationManager.__init__(self)
+
         # Dictionary that keeps count of event listeners by type.
         # This is a bit convoluted for now, but what happens is that
         # scripts tell the FocusTrackingPresenter to listen for
@@ -73,6 +75,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self.setActiveScript(None)
 
+        # Initialize variable to make pylint happy.
+        #
+        self._defaultScript = None
+        self._appStateInfo = None
+        self._listenerCounts = None
+
         if settings.debugEventQueue:
             self._enqueueEventCount = 0
             self._dequeueEventCount = 0
@@ -540,7 +548,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # displayed. See Orca bug #409731 for more details.
             #
             if not event.type.startswith("mouse:"):
-                s = self._getScript(event.source.getApplication())
+                s = self._getScript(event.host_application or \
+                                      event.source.getApplication())
             else:
                 s = orca_state.activeScript
             if not s:
@@ -574,7 +583,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                       + orca_state.activeScript.name)
 
                         self.setActiveScript(
-                            self._getScript(event.source.getApplication()))
+                            self._getScript(event.host_application or \
+                                              event.source.getApplication()))
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
@@ -953,16 +963,21 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # can tell the user about itself.
             #
             class _FakeEvent:
-                pass
+                def __init__(self, source, eventType,
+                             detail1, detail2, any_data):
+                    self.source = source
+                    self.type = eventType
+                    self.detail1 = detail1
+                    self.detail2 = detail2
+                    self.any_data = any_data
+
             class _FakeData:
+                def __init__(self):
+                    pass
                 def value(self):
                     return None
-            fe = _FakeEvent()
-            fe.source   = win
-            fe.type     = "window:activate"
-            fe.detail1  = 0
-            fe.detail2  = 0
-            fe.any_data = _FakeData()
+
+            fe = _FakeEvent(win, "window:activate", 0, 0, _FakeData())
             e = pyatspi.event.Event(fe)
             self._enqueueEvent(e)
 
