@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 2828 2007-09-13 19:26:33Z wwalker $"
-__version__   = "$Revision: 2828 $"
-__date__      = "$Date: 2007-09-13 15:26:33 -0400 (Thu, 13 Sep 2007) $"
+__id__        = "$Id: mag.py 3096 2007-10-18 18:20:18Z wwalker $"
+__version__   = "$Revision: 3096 $"
+__date__      = "$Date: 2007-10-18 14:20:18 -0400 (Thu, 18 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -38,21 +38,20 @@ except:
     pass
 import time
 
-import atspi
+import pyatspi
 import debug
 import settings
 
 _magnifierAvailable = False
 
 try:
-    atspi.ORBit.load_typelib('GNOME_Magnifier')
+    import ORBit
+    ORBit.load_typelib('GNOME_Magnifier')
     import GNOME.Magnifier
     _magnifierAvailable = True
 except:
     pass
 
-import time
-
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -68,7 +67,8 @@ _magnifier = None
 _roiWidth = 0
 _roiHeight = 0
 
-# Minimum values for the center of the ROI
+# Minimum/maximum values for the center of the ROI
+# in source screen coordinates.
 #
 _minROIX = 0
 _maxROIX = 0
@@ -77,19 +77,31 @@ _maxROIY = 0
 
 # The current region of interest.
 #
+# A GNOME.Magnifier.RectBounds object which consists of x1, y1, x2, y2 values
+# that are in source screen coordinates.
+#
 _roi = None
 
-# The area we are magnifying
+# The area of the source display that is not covered by the magnifier.
+#
+# A GNOME.Magnifier.RectBounds object which consists of x1, y1, x2, y2 values
+# that are in source screen coordinates.  If the COMPOSITE support is enabled
+# in gnome-mag, then this typically becomes the entire source screen.  If it
+# it not enabled, however, and the target and source displays are the same,
+# this ends up becoming the portion of the screen that is not covered by the
+# magnifier.
 #
 _sourceDisplayBounds = None
 
-# The area where we are magnifying
+# The area on the target display where we are placing the magnifier.
+#
+# A GNOME.Magnifier.RectBounds object which consists of x1, y1, x2, y2 values
+# that are in target screen coordinates.
 #
 _targetDisplayBounds = None
 
-# The ZoomRegion we care about [[[TODO: WDW - we should be more careful about
-# just what we're doing here.  The Magnifier allows more than one ZoomRegion
-# and we're just picking up the first one.]]]
+# The ZoomRegion we care about.  We only use one ZoomRegion and we
+# make it occupy the whole magnifier.
 #
 _zoomer = None
 
@@ -102,6 +114,10 @@ _lastMouseEventTime = time.time()
 #
 _pollMouseDisabled = False
 
+# Whether or not composite is being used.
+#
+_fullScreenCapable = True
+
 def __setROI(rect):
     """Sets the region of interest.
 
@@ -111,7 +127,11 @@ def __setROI(rect):
 
     global _roi
 
+    debug.println(debug.LEVEL_ALL, "mag.py:__setROI: (%d, %d), (%d, %d)" \
+                  % (rect.x1, rect.y1, rect.x2, rect.y2))
+
     _roi = rect
+
     _zoomer.setROI(_roi)
     _zoomer.markDirty(_roi)  # [[[TODO: WDW - for some reason, this seems
                              # necessary.]]]
@@ -153,22 +173,22 @@ def __setROIPush(x, y):
     - y: integer in unzoomed system coordinates representing y component
     """
 
-    needNewROI = False
+    #needNewROI = False
     newROI = GNOME.Magnifier.RectBounds(_roi.x1, _roi.y1, _roi.x2, _roi.y2)
     if x < _roi.x1:
-        needNewROI = True
+        #needNewROI = True
         newROI.x1 = x
         newROI.x2 = x + _roiWidth
     elif x > _roi.x2:
-        needNewROI = True
+        #needNewROI = True
         newROI.x2 = x
         newROI.x1 = x - _roiWidth
     if y < _roi.y1:
-        needNewROI = True
+        #needNewROI = True
         newROI.y1 = y
         newROI.y2 = y + _roiHeight
     elif y > _roi.y2:
-        needNewROI = True
+        #needNewROI = True
         newROI.y2 = y
         newROI.y1 = y - _roiHeight
 
@@ -228,9 +248,11 @@ def __onMouseEvent(e):
 
     if settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PUSH:
         __setROIPush(x, y)
-    elif settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PROPORTIONAL:
+    elif settings.magMouseTrackingMode == \
+                                settings.MAG_MOUSE_TRACKING_MODE_PROPORTIONAL:
         __setROIProportional(x, y)
-    elif settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_CENTERED:
+    elif settings.magMouseTrackingMode == \
+                                    settings.MAG_MOUSE_TRACKING_MODE_CENTERED:
         __setROICenter(x, y)
 
 def __getValueText(slot, value):
@@ -280,6 +302,7 @@ def applySettings():
     global _maxROIX
     global _maxROIY
     global _pollMouseDisabled
+    global _fullScreenCapable
 
     ########################################################################
     #                                                                      #
@@ -289,32 +312,109 @@ def applySettings():
 
     _magnifier.clearAllZoomRegions()
 
+    # Define where the magnifier will live.
+    #
     try:
         _magnifier.TargetDisplay = settings.magTargetDisplay
     except:
         pass
 
+    # Define what will be magnified.
+    #
     try:
         _magnifier.SourceDisplay = settings.magSourceDisplay
     except:
         pass
 
-    magnifierPBag = _magnifier.getProperties()
+    # Find out where the user wants to place the target display.
+    #
+    prefLeft   = settings.magZoomerLeft
+    prefTop    = settings.magZoomerTop
+    prefRight  = settings.magZoomerRight
+    prefBottom = settings.magZoomerBottom
+    updateTarget = True
 
+    # Find out if we're using composite.
+    #
     try:
+        _fullScreenCapable = _magnifier.fullScreenCapable()
+    except:
+        debug.printException(debug.LEVEL_WARNING)
+
+    magnifierPBag = _magnifier.getProperties()
+    
+    # If we're not using composite, bad things will happen if we allow the
+    # target display to occupy more than 50% of the full screen.  Also, if
+    # it occupies the same space as something that should be magnified (e.g.
+    # the Applications menu), that item will not be magnified. Therefore,
+    # we're going to prefer the right half of the screen for the target
+    # display -- unless gnome-mag is already running and not in "full screen"
+    # mode.
+    #
+    if not _fullScreenCapable \
+       and _magnifier.SourceDisplay == _magnifier.TargetDisplay:
+        # At this point, the target display bounds have not been set. if they
+        # are all 0, then we know that gnome-mag isn't already running.
+        #
+        magAlreadyRunning = False
+        tdb = magnifierPBag.getValue("target-display-bounds").value()
+        if tdb.x1 or tdb.x2 or tdb.y1 or tdb.y2:
+            magAlreadyRunning = True
+
+        # At this point, because we haven't set the target display bounds,
+        # gnome-mag has not modified the source display bounds. Therefore,
+        # we can use the current source display bounds to get the
+        # dimensions of the full screen -- assuming gnome-mag has not
+        # already been launched with a split screen. Comparing the values
+        # of the source and target display bounds lets us know if gnome-mag
+        # has been started in "full screen" mode.
+        #
+        sdb = magnifierPBag.getValue("source-display-bounds").value()
+        magFullScreen = magAlreadyRunning \
+                        and (tdb.x1 == sdb.x1) and (tdb.x2 == sdb.x2) \
+                        and (tdb.y1 == sdb.y1) and (tdb.y2 == sdb.y2)
+
+        sourceArea = (sdb.x2 - sdb.x1) * (sdb.y2 - sdb.y1)
+        prefArea = (prefRight - prefLeft) * (prefBottom - prefTop)
+        if prefArea > sourceArea/2:
+            debug.println(
+                debug.LEVEL_WARNING,
+                "Composite is not being used. The preferred target area is\n"\
+                "greater than 50% of the source area.  These settings can\n"\
+                "render the contents of the screen inaccessible.")
+            if not magAlreadyRunning or magFullScreen:
+                debug.println(
+                    debug.LEVEL_WARNING,
+                    "Setting the target display to the screen's right half.")
+                prefRight  = sdb.x2
+                prefLeft   = sdb.x2/2
+                prefTop    = sdb.y1
+                prefBottom = sdb.y2
+            else:
+                updateTarget = False
+                debug.println(
+                    debug.LEVEL_WARNING,
+                    "Gnome-mag is already running.  Using those settings.")
+
+    # Now, tell the magnifier where we want it to live on the target display.
+    # The coordinates are in screen coordinates of the target display.
+    # This will have the side effect of setting up other stuff for us, such
+    # as source-display-bouinds.
+    #
+    if updateTarget:
         tdb = magnifierPBag.getValue("target-display-bounds").value()
         magnifierPBag.setValue(
             "target-display-bounds",
-            atspi.ORBit.CORBA.Any(
-                atspi.ORBit.CORBA.TypeCode(
+            ORBit.CORBA.Any(
+                ORBit.CORBA.TypeCode(
                     tdb.__typecode__.repo_id),
-                GNOME.Magnifier.RectBounds(settings.magZoomerLeft,
-                                           settings.magZoomerTop,
-                                           settings.magZoomerRight,
-                                           settings.magZoomerBottom)))
-    except:
-        debug.printException(debug.LEVEL_OFF)
+                GNOME.Magnifier.RectBounds(prefLeft,
+                                           prefTop,
+                                           prefRight,
+                                           prefBottom)))
 
+    # Set a bunch of other magnifier properties...
+    #
     if settings.enableMagCursor:
         bonobo.pbclient_set_float(
             magnifierPBag, "cursor-scale-factor", 1.0 * settings.magZoomFactor)
@@ -348,14 +448,14 @@ def applySettings():
     color = magnifierPBag.getValue("cursor-color")
     magnifierPBag.setValue(
         "cursor-color",
-        atspi.ORBit.CORBA.Any(
+        ORBit.CORBA.Any(
             color.typecode(),
             long(colorString, 0)))
 
     color = magnifierPBag.getValue("crosswire-color")
     magnifierPBag.setValue(
         "crosswire-color",
-        atspi.ORBit.CORBA.Any(
+        ORBit.CORBA.Any(
             color.typecode(),
             long(colorString, 0)))
 
@@ -375,24 +475,98 @@ def applySettings():
     #                                                                      #
     ########################################################################
 
-    _sourceDisplayBounds = magnifierPBag.getValue(
-        "source-display-bounds").value()
-
+    # We set the target-display-bounds above, but let's ask gnome-mag for
+    # what it really set it to.  Hopefully, it was the same thing.
+    #
     _targetDisplayBounds = magnifierPBag.getValue(
         "target-display-bounds").value()
 
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier target bounds preferences: (%d, %d), (%d, %d)" \
+                  % (settings.magZoomerLeft, settings.magZoomerTop, \
+                     settings.magZoomerRight, settings.magZoomerBottom))
+
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier target bounds actual: (%d, %d), (%d, %d)" \
+                  % (_targetDisplayBounds.x1, _targetDisplayBounds.y1, \
+                     _targetDisplayBounds.x2, _targetDisplayBounds.y2))
+
+    # If the COMPOSITE support is not enabled in gnome-mag, then the
+    # source-display-bounds will be adjusted to accomodate portion of the
+    # display not being covered by the magnifier (assuming there is only
+    # one display).  Otherwise, the source-display-bounds will be the
+    # entire source screen.
+    #
+    _sourceDisplayBounds = magnifierPBag.getValue(
+        "source-display-bounds").value()
+
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier source bounds actual: (%d, %d), (%d, %d)" \
+                  % (_sourceDisplayBounds.x1, _sourceDisplayBounds.y1, \
+                     _sourceDisplayBounds.x2, _sourceDisplayBounds.y2))
+
+    # If there is nothing we can possibly magnify, then abort our mission.
+    #
+    if ((_sourceDisplayBounds.x2 - _sourceDisplayBounds.x1) == 0) \
+        or ((_sourceDisplayBounds.y2 - _sourceDisplayBounds.y1) == 0):
+        debug.println(
+            debug.LEVEL_SEVERE,
+            "There is nothing to magnify.  This is usually caused\n" \
+            "by a preferences setting that tries to take up the\n" \
+            "full screen for magnification, but the underlying\n"
+            "system does not support full screen magnification.\n"
+            "The causes of that are generally that COMPOSITE\n" \
+            "support has not been enabled in gnome-mag or the\n" \
+            "X Window System Server.  As a result of this issue\n" \
+            "Magnification will not be used.\n")
+        raise RuntimeError, "Nothing can be magnified"
+
+    # Now, we create a zoom region to occupy the whole magnifier (i.e.,
+    # the viewport is in target region coordinates and we make the
+    # viewport be the whole target region).  Note, since we're starting
+    # at (0, 0), the viewportWidth and viewportHeight are the same as
+    # the x2, y2 values for a rectangular region.
+    #
+    viewportWidth = _targetDisplayBounds.x2 - _targetDisplayBounds.x1
+    viewportHeight = _targetDisplayBounds.y2 - _targetDisplayBounds.y1
+
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier zoomer viewport desired: (0, 0), (%d, %d)" \
+                  % (viewportWidth, viewportHeight))
+
+    # Now, let's see what the ROI looks like.
+    #
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier source width: %d (viewport can show %d)" \
+                  % (_sourceDisplayBounds.x2 - _sourceDisplayBounds.x1,
+                   viewportWidth / settings.magZoomFactor))
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier source height: %d (viewport can show %d)" \
+                  % (_sourceDisplayBounds.y2 - _sourceDisplayBounds.y1,
+                   viewportHeight / settings.magZoomFactor))
+
+    # Adjust the ROI in the event the source window is too small for the
+    # target window.  This usually happens when someone expects COMPOSITE
+    # to be enabled, but it isn't.  As a result, they usually have a very
+    # big grey magnifier on their screen.
+    #
+    _roiWidth  = min(_sourceDisplayBounds.x2 - _sourceDisplayBounds.x1,
+                     viewportWidth / settings.magZoomFactor)
+    _roiHeight = min(_sourceDisplayBounds.y2 - _sourceDisplayBounds.y1,
+                     viewportHeight / settings.magZoomFactor)
+
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier zoomer ROI size desired: width=%d, height=%d)" \
+                  % (_roiWidth, _roiHeight))
+
+    # Create the zoomer with a magnification factor, an initial ROI, and
+    # where in magnifier we want it to be (we want it to be in the whole
+    # magnifier).
+    #
     _zoomer = _magnifier.createZoomRegion(
         settings.magZoomFactor, settings.magZoomFactor,
-        GNOME.Magnifier.RectBounds(0,
-                                   0,
-                                   -1,
-                                   -1),
-        GNOME.Magnifier.RectBounds(0,
-                                   0,
-                                   _targetDisplayBounds.x2 \
-                                   - _targetDisplayBounds.x1,
-                                   _targetDisplayBounds.y2 \
-                                   - _targetDisplayBounds.y1))
+        GNOME.Magnifier.RectBounds(0, 0, _roiWidth, _roiHeight),
+        GNOME.Magnifier.RectBounds(0, 0, viewportWidth, viewportHeight))
 
     zoomerPBag = _zoomer.getProperties()
     bonobo.pbclient_set_boolean(zoomerPBag, "is-managed", True)
@@ -420,11 +594,21 @@ def applySettings():
 
     viewport = zoomerPBag.getValue("viewport").value()
 
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier viewport actual: (%d, %d), (%d, %d)" \
+                  % (viewport.x1, viewport.y1, viewport.x2, viewport.y2))
+
     magx = zoomerPBag.getValue("mag-factor-x").value()
     magy = zoomerPBag.getValue("mag-factor-y").value()
 
-    _roiWidth = (viewport.x2 - viewport.x1) / magx
-    _roiHeight = (viewport.y2 - viewport.y1) / magy
+    _roiWidth  = min(_sourceDisplayBounds.x2 - _sourceDisplayBounds.x1,
+                     (viewport.x2 - viewport.x1) / magx)
+    _roiHeight = min(_sourceDisplayBounds.y2 - _sourceDisplayBounds.y1,
+                     (viewport.y2 - viewport.y1) / magy)
+
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier zoomer ROI size actual: width=%d, height=%d)" \
+                  % (_roiWidth, _roiHeight))
 
     _minROIX = _sourceDisplayBounds.x1 + (_roiWidth / 2)
     _minROIY = _sourceDisplayBounds.y1 + (_roiHeight / 2)
@@ -432,6 +616,10 @@ def applySettings():
     _maxROIX = _sourceDisplayBounds.x2 - (_roiWidth / 2)
     _maxROIY = _sourceDisplayBounds.y2 - (_roiHeight / 2)
 
+    debug.println(debug.LEVEL_ALL,
+                  "Magnifier ROI min/max center: (%d, %d), (%d, %d)" \
+                  % (_minROIX, _minROIY, _maxROIX, _maxROIY))
+
     _magnifier.addZoomRegion(_zoomer)
 
     #print "MAGNIFIER PROPERTIES:", _magnifier
@@ -452,17 +640,31 @@ def magnifyAccessible(event, obj, extents=None):
     if not _initialized:
         return
 
+    haveSomethingToMagnify = False
+
     if extents:
         [x, y, width, height] = extents
-    elif event and event.type.startswith("object:text-caret-moved") \
-       and obj.text and (obj.text.caretOffset >= 0):
-        offset = obj.text.caretOffset
-        [x, y, width, height] = obj.text.getCharacterExtents(offset,
-                                                             0) # coord type screen
-    elif obj.extents:
-        extents = obj.extents
-        [x, y, width, height] = [extents.x, extents.y, extents.width, extents.height]
-    else:
+        haveSomethingToMagnify = True
+    elif event and event.type.startswith("object:text-caret-moved"):
+        try:
+            text = obj.queryText()
+            if text and (text.caretOffset >= 0):
+                [x, y, width, height] = \
+                    text.getCharacterExtents(text.caretOffset, 0)
+                haveSomethingToMagnify = True
+        except:
+            haveSomethingToMagnify = False
+
+    if not haveSomethingToMagnify:
+        try:
+            extents = obj.queryComponent().getExtents(0)
+            [x, y, width, height] = \
+                [extents.x, extents.y, extents.width, extents.height]
+            haveSomethingToMagnify = True
+        except:
+            haveSomethingToMagnify = False
+
+    if not haveSomethingToMagnify:
         return
 
     # Avoid jerking the display around if the mouse is what ended up causing
@@ -552,18 +754,18 @@ def init():
 
     try:
         applySettings()
+        pyatspi.Registry.registerEventListener(__onMouseEvent, "mouse:abs")
+
+        _initialized = True
+
+        # Zoom to the upper left corner of the display for now.
+        #
+        __setROICenter(0, 0)
+
+        return True
     except:
-        debug.printException(debug.LEVEL_SEVERE)
-
-    atspi.Registry().registerEventListener(__onMouseEvent, "mouse:abs")
-
-    _initialized = True
-
-    # Zoom to the upper left corner of the display for now.
-    #
-    __setROICenter(0, 0)
-
-    return True
+        _magnifier.dispose()
+        raise
 
 def shutdown():
     """Shuts down the magnifier module.
@@ -580,7 +782,7 @@ def shutdown():
     if not _initialized:
         return False
 
-    atspi.Registry().deregisterEventListener(__onMouseEvent,"mouse:abs")
+    pyatspi.Registry.deregisterEventListener(__onMouseEvent,"mouse:abs")
 
     # Someone might have killed the magnifier on us.  They shouldn't
     # have done so, but we need to be able to recover if they did.
