@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3192 2007-10-26 19:41:18Z eitani $"
-__version__   = "$Revision: 3192 $"
-__date__      = "$Date: 2007-10-26 15:41:18 -0400 (Fri, 26 Oct 2007) $"
+__id__        = "$Id: orca.py 3323 2007-12-02 21:52:07Z wwalker $"
+__version__   = "$Revision: 3323 $"
+__date__      = "$Date: 2007-12-02 16:52:07 -0500 (Sun, 02 Dec 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -69,6 +69,7 @@ from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
 from orca_i18n import _           # for gettext support
+from orca_i18n import Q_          # to provide qualified translatable strings
 
 if settings.debugMemoryUsage:
     import gc
@@ -167,6 +168,10 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
         # Either it's None or it's an invalid remote object.
         oldLocusOfFocus = None
 
+    orca_state.focusHistory = \
+        orca_state.focusHistory[:settings.focusHistoryLength - 1]
+    orca_state.focusHistory.insert(0, oldLocusOfFocus)
+
     orca_state.locusOfFocus = obj
     try:
         app = orca_state.locusOfFocus.getApplication()
@@ -329,7 +334,7 @@ def _isPrintableKey(event_string):
                 and (unicodeString.isalnum() or unicodeString.isspace()
                 or unicodedata.category(unicodeString)[0] in ('P', 'S'))
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoPrintableKey: returning: %s" % reply)
+                  "orca._isPrintableKey: returning: %s" % reply)
     return reply
 
 def _isModifierKey(event_string):
@@ -347,7 +352,7 @@ def _isModifierKey(event_string):
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoModifierKey: returning: %s" % reply)
+                  "orca._isModifierKey: returning: %s" % reply)
     return reply
 
 def _isLockingKey(event_string):
@@ -364,7 +369,7 @@ def _isLockingKey(event_string):
     reply = event_string in lockingKeys \
             and not event_string in settings.orcaModifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoLockingKey: returning: %s" % reply)
+                  "orca._isLockingKey: returning: %s" % reply)
     return reply
 
 def _isFunctionKey(event_string):
@@ -385,7 +390,7 @@ def _isFunctionKey(event_string):
 
     reply = event_string in functionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoFunctionKey: returning: %s" % reply)
+                  "orca._isFunctionKey: returning: %s" % reply)
     return reply
 
 def _isActionKey(event_string):
@@ -402,33 +407,58 @@ def _isActionKey(event_string):
 
     reply = event_string in actionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoActionKey: returning: %s" % reply)
+                  "orca._isActionKey: returning: %s" % reply)
+    return reply
+
+def _isNavigationKey(event_string):
+    """Return an indication of whether this is a navigation (arrow) key
+    or if the user has the Orca modifier key held done.
+
+    Arguments:
+    - event: the event string
+    - modifiers: key modifiers state
+
+    Returns True if this is a navigation key.
+    """
+
+    navigationKeys = [ "Left", "Right", "Up", "Down" ]
+
+    reply = (event_string in navigationKeys) or _orcaModifierPressed
+
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._isNavigationKey: returning: %s" % reply)
     return reply
 
 class KeyEventType:
     """Definition of available key event types."""
 
-    """An alphanumeric or punctuation key event."""
+    # An alphanumeric or punctuation key event.
     PRINTABLE = 'PRINTABLE'
 
-    """A modifier key event."""
+    # A modifier key event.
     MODIFIER = 'MODIFIER'
 
-    """A locking key event."""
+    # A locking key event.
     LOCKING = 'LOCKING'
 
-    """A locking key lock event."""
+    # A locking key lock event.
     LOCKING_LOCKED = 'LOCKING_LOCKED'
 
-    """A locking key unlock event."""
+    # A locking key unlock event.
     LOCKING_UNLOCKED = 'LOCKING_UNLOCKED'
 
-    """A function key event."""
+    # A function key event.
     FUNCTION = 'FUNCTION'
 
-    """An action key event."""
+    # An action key event.
     ACTION = 'ACTION'
 
+    # A navigation key event.
+    NAVIGATION = 'NAVIGATION'
+
+    def __init__(self):
+        pass
+
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -458,16 +488,21 @@ def _keyEcho(event):
     #
     if settings.enableKeyEcho:
 
-        if _isPrintableKey(event_string):
-            if not settings.enablePrintableKeys:
-                return
-            eventType = KeyEventType.PRINTABLE
-
-        elif _isModifierKey(event_string):
+        if _isModifierKey(event_string):
             if not settings.enableModifierKeys:
                 return
             eventType = KeyEventType.MODIFIER
 
+        elif _isNavigationKey(event_string):
+            if not settings.enableNavigationKeys:
+                return
+            eventType = KeyEventType.NAVIGATION
+
+        elif _isPrintableKey(event_string):
+            if not settings.enablePrintableKeys:
+                return
+            eventType = KeyEventType.PRINTABLE
+
         elif _isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
@@ -525,20 +560,36 @@ def _processKeyCaptured(event):
 
     if event.type == 0:
         if _isModifierKey(event.event_string) \
-               or event.event_string == "Return":
-            pass
-        elif event.event_string == "Escape":
-            orca_state.capturingKeys = False
+           or _isLockingKey(event.event_string):
+            return True
         else:
-            # Translators: this is a spoken prompt letting the user know
-            # Orca has recorded a new key combination (e.g., Alt+Ctrl+g)
-            # based upon their input.
+            # We want to capture this event, after first doing a little
+            # clean up.  Eliminate modifiers we're not interested in.
             #
-            speech.speak(_("Key captured: %s. Press enter to confirm.") \
-                         % str(event.event_string))
+            mask = (1 << settings.MODIFIER_ORCA |
+                    1 << pyatspi.MODIFIER_ALT |
+                    1 << pyatspi.MODIFIER_SHIFT |
+                    1 << pyatspi.MODIFIER_CONTROL |
+                    1 << pyatspi.MODIFIER_META2 |
+                    1 << pyatspi.MODIFIER_META3)
+            event.modifiers = event.modifiers & mask
+
+            # We want the keyname rather than the printable character.
+            # If it's not on the keypad, get the name of the unshifted
+            # character. (i.e. "1" instead of "!")
+            #
+            keymap = gtk.gdk.keymap_get_default()
+            entries = keymap.get_entries_for_keycode(event.hw_code)
+            event.event_string = gtk.gdk.keyval_name(entries[0][0])
+            if event.event_string.startswith("KP"):
+                name = gtk.gdk.keyval_name(entries[1][0])
+                if name.startswith("KP"):
+                    event.event_string = name
+
             orca_state.lastCapturedKey = event
     else:
         pass
+
     return False
 
 def _processKeyboardEvent(event):
@@ -608,17 +659,17 @@ def _processKeyboardEvent(event):
         #
         speech.stop()
 
-        # If learn mode is enabled, it will echo the keys.
-        #
-        if not settings.learnModeEnabled:
-            _keyEcho(keyboardEvent)
-
         # We treat the Insert key as a modifier - so just swallow it and
         # set our internal state.
         #
         if isOrcaModifier:
             _orcaModifierPressed = True
 
+        # If learn mode is enabled, it will echo the keys.
+        #
+        if not settings.learnModeEnabled:
+            _keyEcho(keyboardEvent)
+
     elif isOrcaModifier \
         and (keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT):
         _orcaModifierPressed = False
@@ -635,7 +686,7 @@ def _processKeyboardEvent(event):
     consumed = False
     try:
         if orca_state.capturingKeys:
-            _processKeyCaptured(keyboardEvent)
+            consumed = _processKeyCaptured(keyboardEvent)
         else:
             consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
             if (not consumed) and (_currentPresentationManager >= 0):
@@ -847,7 +898,7 @@ def loadUserSettings(script=None, inputEvent=None):
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].activate()
 
-    _showMainWindowGUI()
+    showMainWindowGUI()
 
     httpserver.init()
 
@@ -872,7 +923,7 @@ def _showAppPreferencesGUI(script=None, inputEvent=None):
 
     return True
 
-def _showPreferencesGUI(script=None, inputEvent=None):
+def showPreferencesGUI(script=None, inputEvent=None):
     """Displays the user interace to configure Orca and set up
     user preferences using a GUI.
 
@@ -890,7 +941,7 @@ def _showPreferencesGUI(script=None, inputEvent=None):
 
     return True
 
-def _showMainWindowGUI(script=None, inputEvent=None):
+def showMainWindowGUI(script=None, inputEvent=None):
     """Displays the Orca main window.
 
     Returns True to indicate the input event has been consumed.
@@ -1011,7 +1062,7 @@ def init(registry):
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
-                                   "object:children-changed:")
+                                   "object:children-changed")
 
     # We also want to stop speech when a mouse button is pressed.
     #
@@ -1022,7 +1073,8 @@ def init(registry):
 
     masks = []
     mask = 0
-    while mask <= (1 << pyatspi.MODIFIER_NUMLOCK):
+    # Cover all masks in 8 bits.
+    while mask <= 255:
         masks.append(mask)
         mask += 1
     pyatspi.Registry.registerKeystrokeListener(
@@ -1060,10 +1112,20 @@ def start(registry):
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
+    
+    if settings.cacheValues:
+        pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
     registry.start()
 
 def abort(exitCode=1):
+
+    # We know what we are doing here, so tell pylint not to flag
+    # the _exit method call as a warning.  The disable-msg is
+    # localized to just this method.
+    #
+    # pylint: disable-msg=W0212
+
     os._exit(exitCode)
 
 def timeout(signum=None, frame=None):
@@ -1099,7 +1161,7 @@ def shutdown(script=None, inputEvent=None):
     # Deregister our event listeners
     #
     pyatspi.Registry.deregisterEventListener(_onChildrenChanged,
-                                             "object:children-changed:")
+                                             "object:children-changed")
     pyatspi.Registry.deregisterEventListener(_onMouseButton,
                                              "mouse:button")
 
@@ -1175,27 +1237,92 @@ def abortOnSignal(signum, frame):
 
 def usage():
     """Prints out usage information."""
-    print "Usage: orca [OPTION...]"
+    print _("Usage: orca [OPTION...]")
     print
-    print "-?, --help                   Show this help message"
+
+    # Translators: this is the description of the command line option 
+    # '-?, --help' that is used to display usage information.
+    #
+    print "-?, --help                   " + _("Show this help message")
+
     print "-v, --version                %s" % platform.version
-    print "-s, --setup, --gui-setup     Set up user preferences"
-    print "-t, --text-setup             Set up user preferences (text version)"
-    print "-n, --no-setup               Skip set up of user preferences"
-    print "-u, --user-prefs-dir=dirname",
-    print " Use alternate directory for user preferences"
-    print "-e, --enable=[speech|braille|braille-monitor|magnifier|main-window]",
-    print " Force use of option"
-    print "-d, --disable=" \
-          "[speech|braille|braille-monitor|magnifier|main-window]" \
-          " Prevent use of option"
-    print "-q, --quit                   Quits Orca (if shell script used)"
+
+    # Translators: this is the description of the command line option
+    # '-s, --setup, --gui-setup' that will initially display a GUI dialog 
+    # that would allow the user to set their Orca preferences.
+    #
+    print "-s, --setup, --gui-setup     " + _("Set up user preferences")
+
+    # Translators: this is the description of the command line option 
+    # '-t, --text-setup' that will initially display a list of questions 
+    # in text form, that the user will need to answer, before Orca will 
+    # startup. For this to happen properly, Orca will need to be run 
+    # from a terminal window.
+    #
+    print "-t, --text-setup             " + \
+          _("Set up user preferences (text version)")
+
+    # Translators: this is the description of the command line option 
+    # '-n, --no-setup' that means that Orca will startup without setting 
+    # up any user preferences.
+    #
+    print "-n, --no-setup               " +  \
+          _("Skip set up of user preferences")
+
+    # Translators: by default, Orca expects to find its user preferences
+    # in a directory called .orca under the user's home directory. This
+    # is the description of the command line option 
+    # '-u, --user-prefs-dir=dirname' that allows you to specify an alternate
+    # location for those user preferences.
+    #
+    print "-u, --user-prefs-dir=dirname " + \
+          _("Use alternate directory for user preferences")
+
+    print "-e, --enable=[" \
+        + Q_("option|speech") + "|" \
+        + Q_("option|braille") + "|" \
+        + Q_("option|braille-monitor") + "|" \
+        + Q_("option|magnifier") + "|" \
+        + Q_("option|main-window") + "]",
+
+    # Translators: if the user supplies an option via the '-e, --enable'
+    # command line option, it will be automatically enabled as Orca is
+    # started.
+    #
+    print _("Force use of option")
+
+    print "-d, --disable=[" \
+        + Q_("option|speech") + "|" \
+        + Q_("option|braille") + "|" \
+        + Q_("option|braille-monitor") + "|" \
+        + Q_("option|magnifier") + "|" \
+        + Q_("option|main-window") + "]",
+
+    # Translators: if the user supplies an option via the '-d, --disable'
+    # command line option, it will be automatically disabled as Orca is
+    # started.
+    #
+    print _("Prevent use of option") 
+
+    # Translators: this is the Orca command line option that will quit Orca.
+    # The user would run the Orca shell script again from a terminal window.
+    # If this command line option is specified, the script will quit any
+    # instances of Orca that are already running.
+    #
+    print "-q, --quit                   " + \
+          _("Quits Orca (if shell script used)")
+
     print
-    print "If Orca has not been previously set up by the user, Orca"
-    print "will automatically launch the preferences set up unless"
-    print "the -n or --no-setup option is used."
+
+    # Translators: this is text being sent to a terminal and we want to 
+    # keep the text lines within terminal boundaries.
+    #
+    print _("If Orca has not been previously set up by the user, Orca\n" \
+            "will automatically launch the preferences set up unless\n" \
+            "the -n or --no-setup option is used.")
+
     print
-    print "Report bugs to orca-list@gnome.org."
+    print _("Report bugs to orca-list@gnome.org.")
 
 def main():
     """The main entry point for Orca.  The exit codes for Orca will
@@ -1280,35 +1407,66 @@ def main():
 
             if opt in ("-e", "--enable"):
                 feature = val.strip()
-                if feature == "speech":
+
+                # Translators: this option is for enabling speech synthesis
+                # output.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                if feature == Q_("option|speech"):
                     _commandLineSettings["enableSpeech"] = True
-                elif feature == "braille":
+
+                # Translators: this option is for enabling braille output on
+                # a physical refreshable braille display.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|braille"):
                     _commandLineSettings["enableBraille"] = True
-                elif feature == "braille-monitor":
+
+                # Translators: this option is for enabling a GUI to monitor 
+                # what is being sent to the physical braille display.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|braille-monitor"):
                     _commandLineSettings["enableBrailleMonitor"] = True
-                elif feature == "magnifier":
+
+                # Translators: this option is for enabling screen 
+                # magnification.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|magnifier"):
                     _commandLineSettings["enableMagnifier"] = True
-                elif feature == "main-window":
+
+                # Translators: this option is for enabling the display
+                # of the Orca main window.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|main-window"):
                     _commandLineSettings["showMainWindow"] = True
+
                 else:
                     usage()
-                    os._exit(2)
+                    abort(2)
 
             if opt in ("-d", "--disable"):
                 feature = val.strip()
-                if feature == "speech":
+                if feature == Q_("option|speech"):
                     _commandLineSettings["enableSpeech"] = False
-                elif feature == "braille":
+                elif feature == Q_("option|braille"):
                     _commandLineSettings["enableBraille"] = False
-                elif feature == "braille-monitor":
+                elif feature == Q_("option|braille-monitor"):
                     _commandLineSettings["enableBrailleMonitor"] = False
-                elif feature == "magnifier":
+                elif feature == Q_("option|magnifier"):
                     _commandLineSettings["enableMagnifier"] = False
-                elif feature == "main-window":
+                elif feature == Q_("option|main-window"):
                     _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
-                    os._exit(2)
+                    abort(2)
 
             if opt in ("-s", "--gui-setup", "--setup"):
                 setupRequested = True
@@ -1320,14 +1478,14 @@ def main():
                 bypassSetup = True
             if opt in ("-?", "--help"):
                 usage()
-                os._exit(0)
+                abort(0)
             if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
-                os._exit(0)
+                abort(0)
     except:
         debug.printException(debug.LEVEL_OFF)
         usage()
-        os._exit(2)
+        abort(2)
 
     # Do not run Orca if accessibility has not been enabled.
     # We do allow, however, one to force Orca to run via the
@@ -1370,10 +1528,10 @@ def main():
     # user has bypassed any setup via the --no-setup switch.
     #
     if setupRequested and (not bypassSetup) and showGUI:
-        _showPreferencesGUI()
+        showPreferencesGUI()
     elif (not _userSettings) and (not bypassSetup):
         if desktopRunning:
-            _showPreferencesGUI()
+            showPreferencesGUI()
         else:
             _showPreferencesConsole()
 
