@@ -19,9 +19,9 @@
 
 """Custom script for gedit."""
 
-__id__        = "$Id: gedit.py 3230 2007-11-09 17:02:46Z richb $"
-__version__   = "$Revision: 3230 $"
-__date__      = "$Date: 2007-11-09 12:02:46 -0500 (Fri, 09 Nov 2007) $"
+__id__        = "$Id: gedit.py 3280 2007-11-29 19:55:36Z wwalker $"
+__version__   = "$Revision: 3280 $"
+__date__      = "$Date: 2007-11-29 14:55:36 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -110,14 +110,14 @@ class Script(default.Script):
         """
         return SpeechGenerator(self)
 
-    def __sayAllProgressCallback(self, context, type):
+    def __sayAllProgressCallback(self, context, progressType):
         """Provide feedback during the sayAll operation.
         """
 
-        if type == speechserver.SayAllContext.PROGRESS:
+        if progressType == speechserver.SayAllContext.PROGRESS:
             #print "PROGRESS", context.utterance, context.currentOffset
             pass
-        elif type == speechserver.SayAllContext.INTERRUPTED:
+        elif progressType == speechserver.SayAllContext.INTERRUPTED:
             #print "INTERRUPTED", context.utterance, context.currentOffset
             offset = context.currentOffset
             for i in range(0, len(context.obj)):
@@ -129,7 +129,7 @@ class Script(default.Script):
                 else:
                     text.setCaretOffset(offset)
                     break
-        elif type == speechserver.SayAllContext.COMPLETED:
+        elif progressType == speechserver.SayAllContext.COMPLETED:
             #print "COMPLETED", context.utterance, context.currentOffset
             obj = context.obj[len(context.obj)-1]
             obj.queryText().setCaretOffset(context.currentOffset)
