@@ -19,22 +19,19 @@
 
 """Custom script for planner."""
 
-__id__        = "$Id: planner.py 2397 2007-05-16 17:01:38Z wwalker $"
-__version__   = "$Revision: 2397 $"
-__date__      = "$Date: 2007-05-16 13:01:38 -0400 (Wed, 16 May 2007) $"
+__id__        = "$Id: planner.py 3088 2007-10-18 14:23:52Z shaeger $"
+__version__   = "$Revision: 3088 $"
+__date__      = "$Date: 2007-10-18 10:23:52 -0400 (Thu, 18 Oct 2007) $"
 __copyright__ = "Copyright (c) 2006-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.debug as debug
 import orca.default as default
-import orca.atspi as atspi
-import orca.rolenames as rolenames
-import orca.orca as orca
 import orca.braille as braille
-import orca.speech as speech
 import orca.settings as settings
 import orca.speechgenerator as speechgenerator
 import orca.braillegenerator as braillegenerator
+import pyatspi
 
 from orca.orca_i18n import _ # for gettext support
 
@@ -61,8 +58,10 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(text, 
+                                         self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(text, 
+                                         self._script.getDisplayedText(obj))
 
         # First special toggle button is the one in the toolbar and
         # that it has no name Application should implement an
@@ -70,7 +69,7 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         # speech/braille "display more options" when the focus is in
         # one of these toggle buttons.
         #
-        roleList = [rolenames.ROLE_TOGGLE_BUTTON, rolenames.ROLE_TOOL_BAR]
+        roleList = [pyatspi.ROLE_TOGGLE_BUTTON, pyatspi.ROLE_TOOL_BAR]
 
         if self._script.isDesiredFocusedItem(obj, roleList) and not obj.name:
             text += _("Display more options")
@@ -82,22 +81,26 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         # label that should be associated and its state (checked or
         # not)
         #
-        rolesList = [rolenames.ROLE_TOGGLE_BUTTON,\
-                     rolenames.ROLE_FILLER,\
-                     rolenames.ROLE_FILLER,\
-                     rolenames.ROLE_PANEL,\
-                     rolenames.ROLE_PANEL]
+        rolesList = [pyatspi.ROLE_TOGGLE_BUTTON, \
+                     pyatspi.ROLE_FILLER, \
+                     pyatspi.ROLE_FILLER, \
+                     pyatspi.ROLE_PANEL, \
+                     pyatspi.ROLE_PANEL]
         if self._script.isDesiredFocusedItem(obj, rolesList):
             debug.println(debug.LEVEL_FINEST,
                           "planner.onFocus - main window: " \
                           + "one of the four graphic toggle buttons.")
             filler = obj.parent
-            allLabels = self._script.findByRole(filler, rolenames.ROLE_LABEL)
+            allLabels = self._script.findByRole(filler, pyatspi.ROLE_LABEL)
             text += allLabels[0].name
 
+        if obj.getState().contains(pyatspi.STATE_CHECKED):
+            brailleindicatorindex = 1
+        else:
+            brailleindicatorindex = 0
+
         text = self._script.appendString(
-            settings.brailleRadioButtonIndicators[
-                obj.state.count(atspi.Accessibility.STATE_CHECKED)],
+            settings.brailleRadioButtonIndicators[brailleindicatorindex],
             text)
 
         text = self._script.appendString(text, self._getTextForRole(obj))
@@ -120,24 +123,24 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
     # Also for each one of the grphics buttons in the main window
     #
     def _getSpeechForToggleButton(self, obj, already_focused):
-
-        utterances=[]
-        tmp=[]
+        utterances = []
+        tmp = []
 
         # Application should implement an accessible name in this
         # component, but until this is made We speech/braille "display
         # more options" when the focus is in one of these toggle
         # buttons.
         #
-        roleList=[rolenames.ROLE_TOGGLE_BUTTON,\
-                  rolenames.ROLE_TOOL_BAR]
+
+        roleList = [pyatspi.ROLE_TOGGLE_BUTTON, \
+                    pyatspi.ROLE_TOOL_BAR]
 
         if self._script.isDesiredFocusedItem(obj, roleList) and not obj.name:
             if not already_focused:
                 tmp.append(_("Display more options"))
                 tmp.extend(self._getDefaultSpeech(obj, already_focused))
 
-                if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+                if obj.getState().contains(pyatspi.STATE_CHECKED):
                     tmp.append(_("pressed"))
                 else:
                     tmp.append(_("not pressed"))
@@ -145,7 +148,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 utterances.extend(tmp)
                 utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
-                if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+                if obj.getState().contains(pyatspi.STATE_CHECKED):
                     utterances.append(_("pressed"))
                 else:
                     utterances.append(_("not pressed"))
@@ -157,21 +160,22 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
         # append for each one the button the label that should be
         # associated and its state (checked or not)
         #
-        roleList = [rolenames.ROLE_TOGGLE_BUTTON,\
-                    rolenames.ROLE_FILLER,\
-                    rolenames.ROLE_FILLER,\
-                    rolenames.ROLE_PANEL,\
-                    rolenames.ROLE_PANEL]
+        roleList = [pyatspi.ROLE_TOGGLE_BUTTON, \
+                    pyatspi.ROLE_FILLER, \
+                    pyatspi.ROLE_FILLER, \
+                    pyatspi.ROLE_PANEL, \
+                    pyatspi.ROLE_PANEL]
+
         if self._script.isDesiredFocusedItem(obj, roleList):
             debug.println(debug.LEVEL_FINEST,
                           "planner.onFocus - main window: " \
                           + "one of the four graphic toggle buttons.")
             if not already_focused:
                 filler = obj.parent
-                allLabels = self._script.findByRole(filler, rolenames.ROLE_LABEL)
+                allLabels = self._script.findByRole(filler, pyatspi.ROLE_LABEL)
                 tmp.append(allLabels[0].name)
                 tmp.extend(self._getDefaultSpeech(obj, already_focused))
-                if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+                if obj.getState().contains(pyatspi.STATE_CHECKED):
                     tmp.append(_("pressed"))
                 else:
                     tmp.append(_("not pressed"))
@@ -179,7 +183,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 utterances.extend(tmp)
                 utterances.extend(self._getSpeechForObjectAvailability(obj))
             else:
-                if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+                if obj.getState().contains(pyatspi.STATE_CHECKED):
                     utterances.append(_("pressed"))
                 else:
                     utterances.append(_("not pressed"))
