@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 3223 2007-11-06 21:52:20Z richb $"
-__version__   = "$Revision: 3223 $"
-__date__      = "$Date: 2007-11-06 16:52:20 -0500 (Tue, 06 Nov 2007) $"
+__id__        = "$Id: settings.py 3278 2007-11-29 19:07:04Z shaeger $"
+__version__   = "$Revision: 3278 $"
+__date__      = "$Date: 2007-11-29 14:07:04 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -289,7 +289,7 @@ enableBrailleMonitor    = False
 
 # Strings used to indicate checkbox/radio button states in braille:
 #
-brailleCheckBoxIndicators = ["< >", "<x>"]
+brailleCheckBoxIndicators = ["< >", "<x>", "<->"]
 brailleRadioButtonIndicators = ["& y", "&=y"]
 
 # If True, enable magnification.
@@ -436,7 +436,16 @@ progressBarUpdateInterval = 10
 
 # The complete list of possible text attributes.
 #
-allTextAttributes = "bg-color:; bg-full-height:; bg-stipple:; direction:; editable:; family-name:; fg-color:; fg-stipple:; font-effect:none; indent:0; invisible:; justification:left; language:; left-margin:; line-height:100%; paragraph-style:Default; pixels-above-lines:; pixels-below-lines:; pixels-inside-wrap:; right-margin:; rise:; scale:; size:; stretch:; strikethrough:false; style:normal; text-decoration:none; text-rotation:0; text-shadow:none; underline:none; variant:; vertical-align:baseline; weight:400; wrap-mode:; writing-mode:lr-tb;"
+allTextAttributes = \
+    "bg-color:; bg-full-height:; bg-stipple:; direction:; editable:; " \
+    "family-name:; fg-color:; fg-stipple:; font-effect:none; indent:0; " \
+    "invisible:; justification:left; language:; left-margin:; " \
+    "line-height:100%; paragraph-style:Default; pixels-above-lines:; " \
+    "pixels-below-lines:; pixels-inside-wrap:; right-margin:; rise:; " \
+    "scale:; size:; stretch:; strikethrough:false; style:normal; " \
+    "text-decoration:none; text-rotation:0; text-shadow:none; " \
+    "underline:none; variant:; vertical-align:baseline; weight:400; " \
+    "wrap-mode:; writing-mode:lr-tb;"
 
 # The default set of text attributes to speak to the user. Specific
 # application scripts (or individual users can override these values if
@@ -446,7 +455,9 @@ allTextAttributes = "bg-color:; bg-full-height:; bg-stipple:; direction:; editab
 # it won't be spoken. If no value part is given, then that attribute will
 # always be spoken.
 
-enabledSpokenTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+enabledSpokenTextAttributes = \
+    "size:; family-name:; weight:400; indent:0; underline:none; " \
+    "strikethrough:false; justification:left; style:normal;"
 
 # The default set of text attributes to be brailled for the user. Specific
 # application scripts (or individual users can override these values if
@@ -456,7 +467,9 @@ enabledSpokenTextAttributes = "size:; family-name:; weight:400; indent:0; underl
 # it won't be spoken. If no value part is given, then that attribute will
 # always be brailled.
 
-enabledBrailledTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+enabledBrailledTextAttributes = \
+    "size:; family-name:; weight:400; indent:0; underline:none; " \
+    "strikethrough:false; justification:left; style:normal;"
 
 # Text Attributes Braille Indicator (see textAttributesBrailleIndicator).
 # The values represent the character to be used in the attrOr
@@ -754,4 +767,10 @@ setScriptMapping(re.compile(_('pidgin')), "gaim")
 
 # Show deprecated messeges in debug output.
 # Set this to True to help find potential pyatspi porting problems
+#
 deprecatedMessages = False
+
+# Focus history length. We keep a reference to past focused accessibles to 
+# maximize on caching.
+#
+focusHistoryLength = 5
