@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3222 2007-11-06 20:38:59Z richb $"
-__version__   = "$Revision: 3222 $"
-__date__      = "$Date: 2007-11-06 15:38:59 -0500 (Tue, 06 Nov 2007) $"
+__id__        = "$Id: speechgenerator.py 3278 2007-11-29 19:07:04Z shaeger $"
+__version__   = "$Revision: 3278 $"
+__date__      = "$Date: 2007-11-29 14:07:04 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -220,6 +220,34 @@ class SpeechGenerator:
         else:
             return []
 
+    def _getSpeechForAllTextSelection(self, obj):
+        """Check if this object has text associated with it and it's 
+        completely selected.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        utterance = []
+        try:
+            textObj = obj.queryText()
+        except:
+            pass
+        else:
+            noOfSelections = textObj.getNSelections()
+            if noOfSelections == 1:
+                [string, startOffset, endOffset] = \
+                   textObj.getTextAtOffset(0, pyatspi.TEXT_BOUNDARY_LINE_START)
+                if startOffset == 0 and endOffset == len(string):
+                    # Translators: when the user selects (highlights) text in
+                    # a document, Orca lets them know this.
+                    #
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    utterance = [Q_("text|selected")]
+
+        return utterance
+
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
@@ -271,6 +299,7 @@ class SpeechGenerator:
             name = self._getSpeechForObjectName(obj)
             if name != label:
                 utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
             utterances.extend(self._getSpeechForObjectRole(obj, role))
 
         utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -373,7 +402,11 @@ class SpeechGenerator:
 
         utterances = []
         state = obj.getState()
-        if state.contains(pyatspi.STATE_CHECKED):
+        if state.contains(pyatspi.STATE_INDETERMINATE):
+            # Translators: this represents the state of a checkbox.
+            #
+            checkedState = _("partially checked")
+        elif state.contains(pyatspi.STATE_CHECKED):
             # Translators: this represents the state of a checkbox.
             #
             checkedState = _("checked")
@@ -479,6 +512,10 @@ class SpeechGenerator:
 
         utterances.extend(self._getSpeechForObjectAvailability(obj))
 
+        for child in obj:
+            if child.getRole() == pyatspi.ROLE_TEXT:
+                utterances.extend(self._getSpeechForAllTextSelection(child))
+
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -886,6 +923,8 @@ class SpeechGenerator:
         [text, caretOffset, startOffset] = self._script.getTextLineAtCaret(obj)
         utterances.append(text)
 
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
         self._debugGenerator("_getSpeechForText",
                              obj,
                              already_focused,
